﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.EvolutionTracking {
  [StorableClass]
  [Item("Fragment", "A fragment is an object that represents a piece of genetic information that was transferred from parent to child.")]
  public class Fragment : Item, IFragment {
    [Storable]
    public int Index { get; set; }

    [Storable]
    public int OldIndex { get; set; }

    [Storable]
    private object root;
    public object Root {
      get { return root; }
      set { root = value; }
    }

    protected Fragment(Fragment original, Cloner cloner)
      : base(original, cloner) {
      Root = original.Root;
      Index = original.Index;
      OldIndex = original.OldIndex;
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new Fragment(this, cloner);
    }
    public Fragment() { }
  }

  public class Fragment<T> : Fragment, IFragment<T> where T : class {
    public new T Root {
      get { return (T)base.Root; }
      set { base.Root = value; }
    }
  }
}
