﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.EvolutionTracking {
  [StorableClass]
  [Item("AfterCrossoverOperator", "Performs an action after the crossover operator is applied.")]
  public class BeforeManipulatorOperator<T> : EvolutionTrackingOperator<T>, IManipulatorOperator<T> where T : class,IItem {

    private const string ChildParameterName = "Child";
    public ILookupParameter<T> ChildParameter { get; set; }

    protected BeforeManipulatorOperator(BeforeManipulatorOperator<T> original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new BeforeManipulatorOperator<T>(this, cloner);
    }

    public BeforeManipulatorOperator() {
      ChildParameter = new LookupParameter<T>(ChildParameterName);
      Parameters.Add(ChildParameter);
    }

    public override IOperation Apply() {
      if (GenealogyGraph.Contains(ChildParameter.ActualValue)) {
        var child = ChildParameter.ActualValue;
        var clone = (T)child.Clone();
        var vChild = (IGenealogyGraphNode<T>)GenealogyGraph[child].Last();
        var vClone = new GenealogyGraphNode<T> { Content = clone, Rank = vChild.Rank - 0.5 };
        GenealogyGraph.AddVertex(vClone);
        // adjust parent-child(clone) relationship in the graph
        var parents = vChild.InArcs.Select(a => a.Source);
        vChild.InArcs.Clear();
        foreach (var p in parents) {
          foreach (var a in p.OutArcs) {
            if (a.Target == vChild)
              a.Target = vClone;
          }
          vClone.AddReverseArc(p);
        }
        vChild.AddReverseArc(vClone);
        vClone.AddForwardArc(vChild);
      } else { // this needs to be checked
        var vChild = new GenealogyGraphNode<T> { Content = ChildParameter.ActualValue, Rank = Generations.Value };
        GenealogyGraph.AddVertex(vChild);
      }
      return base.Apply();
    }
  }
}
